#!/bin/sh
RCSID='$Id: dotplot.sh,v 2.54 2002/07/23 18:04:38 schwartz Exp $'
USAGE='usage: dotplot ...'

optint () {
  expr "$1" : "-*[^0-9.-]*\([-0-9.]*\)" \| "0"
  return 0
}

optstr () {
  expr "$1" : "[^=]*=\(.*\)" \| "dotplot"
  return 0;
}

emitprocset () {
cat <<'@'
%%BeginProcSet: PSU_dotplot 1.0 0.0
/DOTPLOT 1240 dict def
/picture-xforms where { pop } { /picture-xforms [] def } ifelse
% XXX - needs to be outside dict begin?
DOTPLOT begin

% for pdf
/URL-base () def
/gensym-counter 0 def

% statusdict begin manualfeed true store end
/units-per-inch 72 def
/scalefactor 1 def
/banner-font-size 0 def
/label-font-size 0 def
/Fontsize 0 def
/basic-arrow-gap 0 def

/left-margin-param 0.7 def
/right-margin-param 1.4 def
/bottom-margin-param 0.7 def
/edge-width-param 1.25 def
/tick-len-param 0.0625 def

/bdef { bind def } bind def
/inch { units-per-inch mul } bdef
/hcin { units-per-inch div } bdef
/point { scalefactor div } bdef

/thin { 0.1 point } bdef
/leftmargin { left-margin-param inch } bdef
/rightmargin { right-margin-param inch } bdef
/bottommargin { bottom-margin-param inch } bdef
/edgewidth { edge-width-param inch } bdef % extra space on left
/ticklen { tick-len-param inch } bdef

/min { 2 copy ge {exch pop} {pop} ifelse } bdef
/max { 2 copy le {exch pop} {pop} ifelse } bdef
/i2s { 20 string cvs } bdef

/concatstrings  % From gs5.01, (c) Aladdin
	{ exch dup length 2 index length add string	% str2 str1 new
	  dup dup 4 2 roll copy		% str2 new new new1
	  length 4 -1 roll putinterval
	} bdef

/gensym-string {
	/gensym-counter gensym-counter 1 add store
	gensym-counter i2s
} bdef

/gensym-title {
	(t) gensym-string concatstrings
} bdef

/i2ks {
	dup 1000 mod 0 eq {
		1000 div cvi i2s (k) concatstrings
	} {
		cvi i2s
	} ifelse
} bdef

/plot-gap {
    plot-gap-pt point
} bdef

/mid { add 2 div } bdef

/inset { % x0 y0 x1 y1 d - x0' y0' x1' y1'
    4 dict begin
        /d exch def
        d sub 4 1 roll
        d sub 4 1 roll
        d add 4 1 roll
        d add 4 1 roll
    end
} bdef

/set-line-width {
    dup 0 eq { pop thin } if
    setlinewidth
} bdef

/CLine { % x0 y0 x1 y1 t c -
  pop
  point set-line-width
  %newpath
  4 2 roll moveto lineto
  %closepath
  stroke
  thin setlinewidth
  Black
} bdef

/Line { % x0 y0 x1 y1 t c -
  setgray
  point set-line-width
  %newpath
  4 2 roll moveto lineto
  %closepath
  stroke
  thin setlinewidth
} bdef

/RLineArrow { % x0 y0 x1 y1 t c -
  % setgray
  pop Black
  gsave
  point set-line-width
  2 setlinejoin
  8 dict begin
    /y1 exch def
    /x1 exch def
    /y0 exch def
    /x0 exch def
    /d 2 point def
    newpath x0 y0 moveto x1 y1 lineto stroke
    newpath
	x1 y1 moveto
	x1 d 2 mul sub y1 d sub lineto
	x1 d 2 mul sub y1 d add lineto
    closepath stroke
  end
  thin setlinewidth
  grestore
} bdef

/LLineArrow { % x0 y0 x1 y1 t c -
  % setgray
  pop Black
  gsave
  point set-line-width
  2 setlinejoin
  8 dict begin
    /y1 exch def
    /x1 exch def
    /y0 exch def
    /x0 exch def
    /d 2 point def
    newpath x0 y0 moveto x1 y1 lineto stroke
    newpath
	x0 y0 moveto
	x0 d 2 mul add y0 d sub lineto
	x0 d 2 mul add y0 d add lineto
    closepath stroke
  end
  thin setlinewidth
  grestore
} bdef

/LLine { % x0 y0 x1 y1 t c -
  % setgray
  pop Black
  pop feature-line-width point set-line-width
  4 copy
  newpath moveto lineto stroke
  pop pop
  newpath feature-circle-radius point 0 360 arc stroke
} bdef

/RLine { % x0 y0 x1 y1 t c -
  % setgray
  pop Black
  pop feature-line-width point set-line-width
  4 copy
  newpath moveto lineto stroke
  newpath feature-circle-radius point 0 360 arc stroke
  pop pop
} bdef

% color fill/stroke rect: baseline, height, thickness, draw op
/CfsRect { % x0 y0 x1 y1 h t op -
  32 dict begin
    /op exch def
    /t exch def
    /h exch def /y1 exch def /x1 exch def /y0 exch def /x0 exch def
    /dx x1 x0 sub def
    /dy y1 y0 sub def
    /dd dx dx mul dy dy mul add sqrt def

    dx 0 eq { /dx thin def } if

    gsave
      t point set-line-width
      x0 y0 translate
      dx 0 ne { dy dx atan } {45} ifelse rotate
        newpath
         0 0 moveto dd 0 rlineto 0 h rlineto dd neg 0 rlineto 0 h neg rlineto
        op
    grestore
  end
  thin setlinewidth
} bdef

/CRect { % x0 y0 x1 y1 h t -
  {stroke} CfsRect
} bdef

/CFRect { % x0 y0 x1 y1 h t -
  {fill} CfsRect
} bdef

/Rect { % x0 y0 x1 y1 h t c -
  setgray
  CRect
} bdef

/Circle { % x y r t c -
  setgray
  point set-line-width
  newpath 0 360 arc closepath stroke
  thin setlinewidth
} bdef

/rect-path {  % x0 y0 x1 y1 -
  newpath
  1 index 3 index % x0 y0 x1 y1  x1 y0
  1 index 3 index % x0 y0 x1 y1  x1 y0 x1 y1
  7 index 1 index % x0 y0 x1 y1  x1 y0 x1 y1 x0 y1
  1 index 5 index % x0 y0 x1 y1  x1 y0 x1 y1 x0 y1 x0 y0
  moveto lineto lineto lineto closepath
  pop pop pop pop
} bdef

/Frame { % x0 y0 x1 y1 t c -
  Black  % XXX
  setgray point set-line-width
  rect-path
  stroke
  thin setlinewidth
} bdef

% --------------------------------------

% On lots of things we want the top to parameters to
% be ``/dir [[stringlist]] '', but for backwards compatability
% we sometimes have them in the reverse order.  So this checks
% to see of top is an array, and if not assumes things need to
% be reversed.
/parameter-type-kludge { % /dir [[stringlist]]
  dup type
  /arraytype ne { exch } if
} bdef

% defaults
/string-params {
  aload length /_len exch def
          % [ str font size sup/sub {color} ]
  _len 0 eq { () tic-font Fontsize 0 {Black} } if
  _len 1 eq {    tic-font Fontsize 0 {Black} } if
  _len 2 eq {             Fontsize 0 {Black} } if
  _len 3 eq {                      0 {Black} } if
  _len 4 eq {                        {Black} } if
  /_color_func exch def
  /_script_offset exch def
  /_font_size exch def
  /_font_name exch def
  /_string exch def
} bdef

% This renders a string in a given font.  Uses global Fontsize.
/show-in-font { % [ str font size sup/sub {color} ]
  8 dict begin
    string-params
    _color_func
    _font_name findfont _font_size scalefont setfont
    /_script_offset _script_offset _font_size 2 3 div mul mul def
    0 _script_offset rmoveto
     _string show
    0 _script_offset neg rmoveto
  end
} bdef

% This computes the width of a string in a given font.
/stringwidth-in-font { % [ str font .. ]
  8 dict begin
    string-params
    gsave
      _font_name findfont _font_size scalefont setfont
      _string stringwidth pop
    grestore
  end
} bdef

% This computes the with of a list of (string * font)s.
/string-width { % [[str font .. ] ...]
  4 dict begin
    /_sum 0 def
    {stringwidth-in-font _sum add /_sum exch def } forall
    _sum
  end
} bdef

/scaled-string-width {
  string-width scalefactor div
} bdef

% ps string version
/scaled-stringwidth {
  stringwidth pop scalefactor div
} bdef

% General parameters for laying out annotations.
/box_height { 0.15 inch } bdef		% the height of an annotation
/row_gap { box_height dup -0.2 mul add } bdef	% space between rows of boxes
/H_row_base {_y1 plot-gap add 0.5 point add } bdef % starting place for rows
/V_row_base {_x0 plot-gap sub 0.5 point sub } bdef

% XXX
/current_row_base 0 def				% height of nth row above base
/box_adjustment 0 def
/current-mode /none def

% Parts of various shapes
/basic_box_top { current_row_base box_height add } bdef
/basic_box_bottom { current_row_base } bdef
/box_top { basic_box_top box_adjustment sub } bdef
/box_bottom { basic_box_bottom box_adjustment add } bdef
/text_bottom { box_top row_gap 5 div add } bdef
/triangle_tip { current_row_base box_height 2 div add } bdef
/arrow_gap { box_height 3 div } bdef
/arrow_bottom { current_row_base arrow_gap add } bdef
/arrow_top { current_row_base arrow_gap dup add add } bdef
/arrowhead-height { box_height 3 div } bdef

% This draws a horizontal banner in the current row.
/Hbanner { % [[...]]  -
 gsave 8 dict begin
   Black /Fontsize banner-font-size def
   _x0 box_bottom moveto
   savedmatrix setmatrix
     Hadjust { show-in-font } forall
 end grestore
} bdef

% These record if anything stuck off the edges of the dotplot. XXX
/Hundershoot false def
/Hovershoot false def

% This adjusts the coordinates so they do not overshoot the plot.
/Hnormalize { % x0 x1 - x0' x1'
  /Hn_x1 exch def
  /Hn_x0 exch def
  Hn_x0 _x0 lt { /Hn_x0 _x0 def /Hundershoot true def } if
  Hn_x1 _x1 gt { /Hn_x1 _x1 def /Hovershoot true def } if
  Hn_x0 Hn_x1
} bdef

/H_render_label_internal {
   savedmatrix setmatrix Hadjust
   { show-in-font } forall
} bdef

/H_render_label {
   Black H_render_label_internal
} bdef

/CH_render_label {
   H_render_label_internal
} bdef

/H_render_other_label {
   Black savedmatrix setmatrix Hotheradjust
   { show-in-font } forall
} bdef

% This renders a label above the current row.
/Hlabel {
  % x0 x1 /dir [[str font] ...] - x0 x1 /dir
  16 dict begin
    gsave
    /Hl_strlist exch def
    /Hl_dir exch def
    /Hl_x1 exch def
    /Hl_x0 exch def

    /Fontsize label-font-size def

    Hl_x0 Hl_x1 Hnormalize mid
      /Hl_mid exch def % (x0+x1)/2
    Hl_strlist scaled-string-width 2 div
      /Hl_radius exch def % width/2

    Hl_mid Hl_radius sub text_bottom moveto

    Hl_mid Hl_radius sub  Hl_mid Hl_radius add {
      Hl_strlist CH_render_label
    }ifstrictHlabel

    Hl_x0 Hl_x1 Hl_dir
    grestore
  end
} bdef

/Hotherlabel {
 % x0 x1 /dir [[str font] ...] - x0 x1 /dir
 /Hl_strlist exch def
 /Hl_dir exch def
 /Hl_x1 exch def
 /Hl_x0 exch def
 8 dict begin gsave
   /Fontsize label-font-size def
   Hl_x1 Hl_x0 mid text_bottom moveto
   Hl_strlist H_render_other_label
 grestore end
 Hl_x0 Hl_x1 Hl_dir
} bdef


% Vertical versions of the above.  XXX
/Vundershoot false def
/Vovershoot false def

/Vnormalize {
  /Vn_y1 exch def
  /Vn_y0 exch def
  Vn_y0 _y0 lt { /Vn_y0 _y0 def /Vundershoot true def } if
  Vn_y1 _y1 gt { /Vn_y1 _y1 def /Vovershoot true def } if
  Vn_y0 Vn_y1
} bdef

/Vbanner {
 % strlist -
 gsave 8 dict begin
   Black /Fontsize banner-font-size def
   box_bottom inverted-features {_y1}{_y0} ifelse moveto
   savedmatrix setmatrix
     Vadjust { show-in-font } forall
 end grestore
} bdef

/V_render_label {
   Black savedmatrix setmatrix Vadjust
   { show-in-font } forall
} bdef

/V_render_other_label {
   Black savedmatrix setmatrix Votheradjust
   { show-in-font } forall
} bdef

/Vlabel {
 % y0 y1 /foo str - y0 y1 /foo
 /Vl_strlist exch def
 /Vl_dir exch def
 /Vl_y1 exch def
 /Vl_y0 exch def
 gsave 8 dict begin
   /Fontsize label-font-size def
   text_bottom
   Vl_y0 Vl_y1 Vnormalize add			% (v0+v1
     Vl_strlist scaled-string-width
     inverted-features {add} {sub} ifelse		% +- width)
     2 div 					% /2
   moveto
   Vl_strlist V_render_label
 end grestore
 Vl_y0 Vl_y1 Vl_dir
} bdef

/Votherlabel {
 % y0 y1 /foo str - y0 y1 /foo
 /Vl_strlist exch def
 /Vl_dir exch def
 /Vl_y1 exch def
 /Vl_y0 exch def
 8 dict begin
 /Fontsize label-font-size def
 gsave text_bottom Vl_y0 Vl_y1 mid
        Fontsize 4 div scalefactor div inverted-features {add}{sub} ifelse
        moveto
	Vl_strlist dup
	scaled-string-width 0 rmoveto
	V_render_other_label grestore
 end
 Vl_y0 Vl_y1 Vl_dir
} bdef

% Various shape annotations
/Htriangle { % x0 x1 /dir str
 {
  Hlabel
  % x0 x1 /dir
  /right eq {
    newpath
    exch dup current_row_base moveto box_top lineto triangle_tip lineto
    closepath
  } {
    newpath
    dup current_row_base moveto box_top lineto triangle_tip lineto
    closepath
  } ifelse
  gsave fill grestore
  gsave Black stroke grestore
 } Hstrict
} bdef

/Vtriangle {
  % y0 y1 /dir str
  Vlabel
  /right eq {
    newpath
    exch dup
    current_row_base exch moveto box_top exch lineto triangle_tip exch lineto
    closepath
  } {
    newpath
    dup
    current_row_base exch moveto box_top exch lineto triangle_tip exch lineto
    closepath
  } ifelse
  gsave fill grestore
  gsave Black stroke grestore
} bdef

/Harrow_basic {
  gsave 8 dict begin
    % Black % XXX - ???
    % x0 x1 /dir
    /dir exch def
    /right dir eq
	{ 1 index arrow_gap translate }
	{ 0 index arrow_gap translate -1 1 scale }
    ifelse
    exch sub /_arrowbody_len exch def

    newpath
    /undir dir ne {
      /_arrowhead_len { 1 16 div inch } def
        _arrowbody_len triangle_tip moveto
        _arrowhead_len neg arrowhead-height 2 div rlineto
        0 arrowhead-height neg rlineto
      closepath fill
    } if
    0 triangle_tip moveto _arrowbody_len triangle_tip lineto stroke
  end grestore
} bdef

/plot-gap-inset { % x0 y0 x1 y1 - x0' y0' x1' y1'
  plot-gap neg inset
} bdef

/plotdata-frame {
  _x0 _y0 _x1 _y1  plot-gap-inset
} bdef

/clip-to-rect { % x0 y0 x1 y1 -
  rect-path clip
} bdef

/clip-none {
    newpath 0 0 moveto closepath clip
    newpath 0 0 moveto
} bdef

/strictly-inside { % x0 x1 - bool
  4 dict begin
    /x1 exch def
    /x0 exch def
    x0 _x1 gt
    x1 _x1 gt
    x1 _x0 lt
    x0 _x0 lt
    or
    or
    or
    not
    % current_row_base 0 ne or % XXX?
  end
} bdef

/ifstrictHlabel { % x0 x1 {} -
  gsave
  3 1 roll
  strictly-inside not
  strict-labels and
  { clip-none } if
  exec
  grestore
} bdef

/Hstrict { % x0 x1 /dir str {} - x0 x1 /dir str
  gsave
  4 index 4 index strictly-inside not
  strict-features and
  { clip-none } if
  exec
  grestore
} bdef

/Harrow { % x0 x1 /dir str
  { Hotherlabel
    Harrow_basic
  } Hstrict
} bdef

/Hthinarrow {
  { Hlabel
    Harrow_basic
  } Hstrict
} bdef

/Hthickarrow { % x0 x1 dir str
  {
    Hlabel
    /right eq {
      newpath
      exch
      dup arrow_bottom moveto arrow_top lineto
      dup
        box_height sub dup dup
  	arrow_top lineto
  	box_top lineto
  	exch
        triangle_tip lineto
      dup
        box_bottom lineto
        arrow_bottom lineto
      closepath
    } {
      newpath
      dup arrow_bottom moveto arrow_top lineto
      dup
        box_height add dup dup
     	arrow_top lineto
  	box_top lineto
  	exch
        triangle_tip lineto
      dup
        box_bottom lineto
        arrow_bottom lineto
      closepath
    } ifelse
    gsave fill grestore
    gsave Black stroke grestore
  } Hstrict
} bdef

/Varrow_basic {
  gsave 8 dict begin
    Black
    % y0 y1 /dir
    /dir exch def
    /right dir eq
	{ arrow_gap 2 index translate }
	{ arrow_gap 1 index translate 1 -1 scale }
    ifelse
    exch sub /_arrowbody_len exch def

    newpath
    /undir dir ne {    
      /_arrowhead_len { 1 16 div inch } def
        triangle_tip _arrowbody_len moveto
        arrowhead-height 2 div _arrowhead_len neg rlineto
        arrowhead-height neg 0 rlineto
      closepath fill
    } if
    triangle_tip 0 moveto triangle_tip _arrowbody_len lineto stroke
  end grestore
} bdef

/Varrow {  % y0 y1 /dir str
  Votherlabel
  Varrow_basic
} bdef

/Vthinarrow {
  Vlabel
  Varrow_basic
} bdef

/Vthickarrow { % y0 y1 /dir str
  Vlabel
  /right eq {
    newpath
    exch % y1 y0
    dup
      arrow_bottom exch moveto	% a_b y0 moveto
      arrow_top exch lineto   	% a_t y0 lineto
    dup
      box_height sub dup dup  	% y1 y1-b_h y1-b_h y1-b_h
	arrow_top exch lineto 	% a_t (y1-b_h) lineto
	box_top exch lineto	% b_t (y1-b_h) lineto
	exch                  	% y1-b_h y1
      triangle_tip exch lineto 	% d_t y1 lineto
    dup                       	% y1-b_h y1-b_h
      box_bottom exch lineto  	% b_b y1-b_h lineto
      arrow_bottom exch lineto	% a_b y1-b_h lineto
    closepath
  } {
    newpath
    dup
      arrow_bottom exch moveto	% a_b y0 moveto
      arrow_top exch lineto   	% a_t y0 lineto
    dup
      box_height add dup dup  	% y1 y1-b_h y1-b_h y1-b_h
	arrow_top exch lineto 	% a_t (y1-b_h) lineto
	box_top exch lineto	% b_t (y1-b_h) lineto
	exch                  	% y1-b_h y1
      triangle_tip exch lineto 	% d_t y1 lineto
    dup                       	% y1-b_h y1-b_h
      box_bottom exch lineto  	% b_b y1-b_h lineto
      arrow_bottom exch lineto	% a_b y1-b_h lineto
    closepath
  } ifelse
  gsave fill grestore
  gsave Black stroke grestore
} bdef

/Hvoid { % x0 x1 str
  /foo exch
  Hlabel pop
  pop pop
} bdef

/Vvoid { % x0 x1 str
  /foo exch
  Vlabel pop
  pop pop
} bdef

/Hbox { % x0 x1 str
  /foo exch
  {
    Hlabel pop
    % x0 x1
    newpath
    exch
    dup current_row_base moveto box_top lineto
    dup box_top lineto current_row_base lineto
    closepath
    gsave fill grestore
    gsave Black stroke grestore
  } Hstrict
} bdef

/Vbox {
  /foo exch Vlabel pop
  % x0 x1
  newpath
  exch
  dup current_row_base exch moveto box_top exch lineto
  dup box_top exch lineto current_row_base exch lineto
  closepath
  gsave fill grestore
  gsave Black stroke grestore
} bdef

/Harrowbox {
  {
    Hlabel
    /right eq {
      newpath
      /Harrowbox_1 exch def
      /Harrowbox_0 exch def
      Harrowbox_0 dup box_bottom moveto box_top lineto
      Harrowbox_1 dup
        box_height 4 div sub Harrowbox_0 max
        dup
  	box_top lineto
  	exch triangle_tip lineto
        box_bottom lineto
      closepath
    } {
      newpath
      /Harrowbox_0 exch def
      /Harrowbox_1 exch def
      Harrowbox_0 dup box_bottom moveto box_top lineto
      Harrowbox_1 dup
        box_height 4 div add Harrowbox_0 min
        dup
  	box_top lineto
  	exch triangle_tip lineto
        box_bottom lineto
      closepath
    } ifelse
    gsave fill grestore
    gsave thin setlinewidth Black stroke grestore
  } Hstrict
} bdef

/Varrowbox { % y0 y1 /dir str
  Vlabel
  /right eq {
    newpath
    /Varrowbox_1 exch def
    /Varrowbox_0 exch def
    Varrowbox_0 dup
      box_bottom exch moveto
      box_top exch lineto
    Varrowbox_1 dup
      box_height 4 div sub Varrowbox_0 max dup box_top exch lineto exch
      triangle_tip exch lineto
      box_bottom exch lineto
    closepath
  } {
    newpath
    /Varrowbox_0 exch def
    /Varrowbox_1 exch def
    Varrowbox_0 dup
      box_bottom exch moveto
      box_top exch lineto
    Varrowbox_1 dup
      box_height 4 div add Varrowbox_0 min dup box_top exch lineto exch
      triangle_tip exch lineto
      box_bottom exch lineto
    closepath
  } ifelse
  gsave fill grestore
  gsave thin setlinewidth Black stroke grestore
} bdef


%  Mode switching commands.  Horizontal puts you in horizontal
%  mode, ready to lay out the horizontal annotations.  Similarly
%  for vertical.

/setrow { box_height row_gap add mul /current_row_base exch def } bdef
/setsize { 1.0 exch sub box_height mul 2 div /box_adjustment exch def } bdef

/setup-bars { % ishorizontal --
    pop
    /Bars { 0.2 setgray } def
    /Bars0 { 0.2 setgray } def
    /Bars1 { 0.4 setgray } def
    /Bars2 { 0.8 setgray } def
} bdef

/horizontal-functions-def {
  /URLArrow { v-url-box Harrow Black } bdef
  /URLThinArrow { h-url-box Hthinarrow Black } bdef
  /Arrow { parameter-type-kludge Harrow Black } bdef
  /ArrowBox { parameter-type-kludge Harrowbox 1 setsize Black } bdef
  /Triangle { parameter-type-kludge Htriangle Black } bdef
  /ThickArrow { parameter-type-kludge Hthickarrow Black } bdef
  /ThinArrow { parameter-type-kludge Hthinarrow Black } bdef
  /LowThinArrow {
        parameter-type-kludge
        8 dict begin
        /text_bottom  { current_row_base box_height 0.99 mul add } def % XXX
        /triangle_tip { current_row_base box_height 0.44 mul add } def % XXX
        /arrow_gap 0 def
        Hthinarrow
        end
  } bdef
  /LowThinArrowOrig {
	parameter-type-kludge
	8 dict begin
        /text_bottom { current_row_base box_height 9 div 2 mul add } def
        /triangle_tip { current_row_base box_height 9 div add } def
        /arrow_gap 0 def
	Hthinarrow
	end
	Black
  } bdef
  /Banner { Hbanner Black } bdef
  /Box { Hbox Black } bdef
  /Void { Hvoid Black } bdef
} bdef

/horizontal {
  main
  grestore gsave
  bordermatrix setmatrix
  0 H_row_base translate
  /horizontalmatrix matrix currentmatrix def

  clip-features {
	newpath
	_x0 -1 inch moveto
	_x0 4 inch lineto
	_x1 4 inch lineto
	_x1 -1 inch lineto
	closepath clip
  } if

  horizontal-functions-def
  true setup-bars
  0 setrow
} bdef

/vertical-functions-def {
  /URLArrow { v-url-box Varrow Black } bdef
  /ArrowBox { parameter-type-kludge Varrowbox 1 setsize } bdef
  /Arrow { parameter-type-kludge Varrow } bdef
  /Triangle { parameter-type-kludge Vtriangle } bdef
  /ThickArrow { parameter-type-kludge Vthickarrow } bdef
  /ThinArrow { parameter-type-kludge Vthinarrow } bdef
  /Banner { Vbanner } bdef
  /Box { Vbox } bdef
  /Void { Vvoid } bdef
  /LowThinArrow {
        parameter-type-kludge
        8 dict begin
        /text_bottom  { current_row_base box_height 0.99 mul add } def % XXX
        /triangle_tip { current_row_base box_height 0.44 mul add } def % XXX
        /arrow_gap 0 def
        Vthinarrow
        end
  } bdef
  /LowThinArrowOrig {
	parameter-type-kludge
	8 dict begin
        /text_bottom { current_row_base box_height 9 div 2 mul add } def
        /triangle_tip { current_row_base box_height 9 div add } def
        /arrow_gap 0 def
	Vthinarrow
	end
  } bdef
} bdef


/vertical {
  main
  grestore gsave
  bordermatrix setmatrix
  inverted-features {
	  _x0 _y0 translate 1 -1 scale 0 height neg translate
	  _x0 neg _y0 neg translate } if

  V_row_base 0 translate -1 1 scale
  /verticalmatrix matrix currentmatrix def

  clip-features {
	newpath
  	4 inch _y0 moveto
  	4 inch _y1 lineto
  	-1 inch _y1 lineto
  	-1 inch _y0 lineto
	closepath clip
  } if
  vertical-functions-def
  false setup-bars
  0 setrow
} bdef

% Switch to data plotting mode.
/plotdata-noclip {
  main
  grestore gsave
  bordermatrix setmatrix
  inverted-dotplot {
	  _x0 _y0 translate 1 -1 scale 0 height neg translate
	  _x0 neg _y0 neg translate } if
  report-plot-once
  newpath
} bdef

/plotdata {
  plotdata-noclip clip-to-selection
} bdef

/do-report-plot {
	/picture-xforms
		picture-xforms aload length  % m1 m2 ... mi i
		matrix currentmatrix matrix defaultmatrix
		matrix invertmatrix matrix concatmatrix
		dup /my-xform exch def
		exch 1 add array astore
	store
	(xform )print my-xform ==
	_x1 _y1 my-xform transform _x0 _y0 my-xform transform
        (bbox )print % x0 y0 x1 y1
	exch i2s print ( )print i2s print ( )print
	exch i2s print ( )print i2s print (\n)print
} bdef

/need-report-plot true def % XXX
/report-plot-once {
	need-report-plot {
		/need-report-plot false store
		do-report-plot
	} if
} bdef

%  Utility variables
/Fontsize 11 def
/Null [[() /Courier]] def  % empty label

% Colors
/Black       { 0 setgray } bdef
/White       { 1 setgray } bdef
/Gray        { 0.5 setgray } bdef
/LightGray   { 0.9 setgray } bdef
/DarkGray    { 0.5 setgray } bdef

/LightRed    { 1.0 0.7 0.7 setrgbcolor } bdef
/Red         { 1.0 0.5 0.5 setrgbcolor } bdef
/DarkRed     { 1.0 0.0 0.0 setrgbcolor } bdef % XXX
/LRed        { LightRed } bdef

/LightGreen  { 0.7 1.0 0.7 setrgbcolor } bdef
/Green       { 0.5 1.0 0.5 setrgbcolor } bdef
/DarkGreen   { 0.0 1.0 0.0 setrgbcolor } bdef % XXX
/LGreen      { LightGreen } bdef

/LightBlue   { 0.7 0.7 1.0 setrgbcolor } bdef
/Blue        { 0.5 0.5 1.0 setrgbcolor } bdef
/DarkBlue    { 0.0 0.0 1.0 setrgbcolor } bdef % XXX
/LBlue       { LightBlue } bdef

/LightYellow { 1.0 1.0 0.7 setrgbcolor } bdef
/Yellow      { 1.0 1.0 0.5 setrgbcolor } bdef
/DarkYellow  { 1.0 1.0 0.0 setrgbcolor } bdef % XXX
/LYellow     { LightYellow } bdef

/Pink        { 1.0 0.0 1.0 setrgbcolor } bdef
/LightPink   { 1.0 0.8 1.0 setrgbcolor } bdef
/DarkPink    { 0.6 0.0 0.6 setrgbcolor } bdef

/Cyan        { 0.0 1.0 1.0 setrgbcolor } bdef
/LightCyan   { 0.8 1.0 1.0 setrgbcolor } bdef
/DarkCyan    { 0.0 0.6 0.6 setrgbcolor } bdef

/LightPurple { 0.769 0.500 0.941 sethsbcolor } bdef
/Purple      { 0.769 0.650 0.941 sethsbcolor } bdef
/DarkPurple  { 0.769 0.867 0.941 sethsbcolor } bdef % XXX
/LPurple     { LightPurple } bdef

/LightOrange { 0.108 0.5 1.0 sethsbcolor } bdef
/Orange      { 0.108 0.8 1.0 sethsbcolor } bdef
/DarkOrange  { 0.108 1.0 1.0 sethsbcolor } bdef % XXX
/LOrange     { LightOrange } bdef

% /White     { 0.9 0.9 0.9 setrgbcolor } bdef

% Clip to the box around the plot.
/clip-to-selection {
  plotdata-frame 1 point inset clip-to-rect
  newpath 0 0 moveto
} bdef

/set-tic-font {
  tic-font findfont tic-font-size scalefont setfont
} bdef

/set-eps-mode {
  %(ticklen )print ticklen =
  %(plot-gap )print plot-gap =
  %(tic-font-size )print tic-font-size =
  /left-margin-param 0 def
  /right-margin-param 0 def
  /bottom-margin-param ticklen 3 mul plot-gap add hcin def
  /edge-width-param 1.25 def
} bdef

/set-main-vars {
  /_x0 x0 def /_y0 y0 def /_x1 x1 def /_y1 y1 def

  % -- savedmatrix is the initial coordinate system
  %    we use it to render text
  /savedmatrix matrix currentmatrix def
  /custom-pattern false def

  /saved-screen [ currentscreen ] def

  /Times-Roman findfont Fontsize scalefont setfont

  /width _x1 _x0 sub def /height _y1 _y0 sub def

  % Decide on landscape or portrait mode based on width/height.
  page-orientation 0 eq {
    width height ge
      { /page-orientation 1 def }
      { /page-orientation 2 def }
    ifelse
  } if

  noauto {
    /_page_y1  11 inch def
    /_page_y0  0 def
    /_page_x1  8.5 inch def
    /_page_x0  0 def
  } {
    clippath pathbbox
    /_page_y1 exch def
    /_page_x1 exch def
    /_page_y0 exch def
    /_page_x0 exch def
  } ifelse

  /_page_dy _page_y1 _page_y0 sub def
  /_page_dx _page_x1 _page_x0 sub def

  page-orientation 1 eq {
    % Landscape mode
    /__landscape true def
    _page_dx 0 translate 90 rotate
    /pagewidth  _page_dy edgewidth sub leftmargin sub rightmargin sub def
    /pageheight _page_dx edgewidth sub leftmargin sub rightmargin sub def
    /Hadjust { 90 rotate } def
    /Hotheradjust { 180 rotate 0 Fontsize 4 div neg rmoveto } def
    /Vadjust { 180 rotate } def
    /Votheradjust { 90 rotate } def
    /HP { gsave savedmatrix setmatrix 90 rotate set-tic-font show grestore } def
  } {
    % Portrait mode
    /__landscape false def
    /pagewidth  _page_dx edgewidth sub leftmargin sub rightmargin sub def
    /pageheight _page_dy edgewidth sub leftmargin sub rightmargin sub def
    /Hadjust { 0 rotate } def
    /Vadjust { 90 rotate } def
    /Hotheradjust { 90 rotate 0 Fontsize 4 div neg rmoveto } def
    /Votheradjust { 0 rotate } def
    /HP { gsave savedmatrix setmatrix set-tic-font show grestore } def
  } ifelse
} def

% -------------------------------------------------------------------------
% entry point here

/need-main true def % XXX
/main {
	need-main {
		/need-main false store
		__main
	} if
} bdef
/__main {
  user-options

  %-
  %(plot )print
  %x0 i2s print ( )print
  %y0 i2s print ( )print
  %x1 i2s print ( )print
  %y1 i2s print (\n)print
  %-

  % XXX - ensure nonempty rectangle
  x1 x0 le { /x1 x0 99 add def } if
  y1 y0 le { /y1 y0 99 add def } if

  set-main-vars
  eps-flag { set-eps-mode } if

  % Offset to avoid margins and such
  %edgewidth margin add dup neg translate
  leftmargin edgewidth add bottommargin translate

  % Offset for page location
  x-offset y-offset translate
  %-
  %(left )print leftmargin edgewidth add i2s print
  %( bottom )print bottommargin i2s print (\n)print
  %(offset )print x-offset i2s print ( )print y-offset i2s print (\n)print
  %-

  % Print title
  banner-line {
    0 pageheight moveto edgewidth dup neg exch rmoveto
       current-directory show
    0 pageheight moveto edgewidth dup neg exch rmoveto 0 Fontsize neg rmoveto
       description show (   ) show date show
  } if

  % Scale to dot-plot sizes
  /tmpwidth override-width 0 ne {override-width} {width} ifelse def
  /tmpheight override-height 0 ne {override-height} {height} ifelse def
  /scalefactor
    override-scale 0 ne
	{72 override-scale div}
	{pagewidth tmpwidth div pageheight tmpheight div min}
    ifelse
  def
  scalefactor dup scale
  %-%(scale )print scalefactor =
  
  /units-per-inch 72 scalefactor div def
  thin setlinewidth

  % Move origin to upper left of selected region
  _x0 neg _y0 neg translate

  % -- bordermatrix is erect, but scaled to fit the data
  %    we use it to render horizontal annotations and tickmarks
  /bordermatrix matrix currentmatrix def

  % Draw box for axis, leave margin around the plotdata
  plotdata-frame 1 72 div 0 Frame
  %plotdata-frame thin 0 Frame % XXX ???

  % Draw ticks
  /basicticstep { % x - x'
	  % log round 1 sub 10 exch exp cvi
	  log 1 sub truncate 10 exch exp
	  1 max cvi
  } bdef

  /xtic { % x y -
	  2 copy	% x y x y
	  ticklen sub cvi % x y x y'
	  0 0 Line
  } bdef

  /xtic-small { % x y -
	  2 copy	% x y x y
	  ticklen 2 div sub cvi % x y x y'
	  0 0 Line
  } bdef

  /ytic { % y x -
	  2 copy	% y x y x
	  ticklen add cvi	% y x y x'
	  4 -1 roll	% x y x' y
	  0 0 Line
  } bdef

  /ytic-small { % y x -
	  2 copy	% y x y x
	  ticklen 2 div add cvi	% y x y x'
	  4 -1 roll	% x y x' y
	  0 0 Line
  } bdef

  /x-with-bias {
	  xticbias add
	  xbias sub cvi
  } bdef

  /x-with-k {
	  x-with-bias
	  k-flag { i2ks } { i2s } ifelse
  } bdef

  /xlabel { % x y
      gsave 4 dict begin
	  set-tic-font /Fontsize tic-font-size def
	  2 copy moveto				% x y
	  1 index
	    x-with-k
	    stringwidth pop
	    2 div scalefactor div neg	% x y -xlen
	  ticklen -3 mul 				% x y -xlen -3ylen
	  rmoveto					% x y
	  pop
	  x-with-k
	  HP
      end grestore
  } bdef

  /xlabel+tic {
    2 copy  xtic xlabel
  } bdef

  /ylabel { % y x
      gsave 4 dict begin
	  set-tic-font /Fontsize tic-font-size def
	  exch 2 copy moveto			% x y
	  ticklen 0 rmoveto exch			% y x
	  pop y-scale-fudge div cvi
	  percent-flag
		{ i2s (%) concatstrings }
	  	{ k-flag { i2ks } { i2s } ifelse }
	  ifelse
	  0 Fontsize 4 div scalefactor div inverted-dotplot not { neg } if
	  rmoveto
	  HP
      end grestore
  } bdef

  /ylabel+tic {
    2 copy  ytic ylabel
  } bdef

  /x-tic-step {
    htic-step 0 eq { width basicticstep } { htic-step } ifelse
  } bdef

  /x-label-step {
    hlab-step 0 eq { xstep } { hlab-step } ifelse
  } bdef

  /y-tic-step {
    vtic-step 0 eq { height basicticstep } { vtic-step } ifelse
  } bdef

  /y-label-step {
    vlab-step 0 eq { ystep } {h lab-step } ifelse
  } bdef

  /xstep x-tic-step cvi def
  /ystep y-tic-step cvi def
  /xlabstep x-label-step cvi def
  /ylabstep y-label-step cvi def
  /xstart _x0 dup xstep mod sub xstep add def
  /ystart _y0 dup ystep mod sub ystep add def
  /xend _x1 dup xstep mod sub xstep sub def
  /yend _y1 dup ystep mod sub ystep sub def

  /within-n-of { % x n y -- x is within n of y
          3 -1 roll  % n y x
          sub abs    % n d
          gt
  } bdef

  /goodxtic { % x - not too close to endpoints
          xstep 10 div     % x s
          2 copy           % x s x s
          _x0 within-n-of  % x s b
	  3 1 roll         % b x s
          _x1 within-n-of  % b b
          or not
  } bdef

  /goodxlabel { % x - isgood
      gsave 8 dict begin
          /x exch def
	  set-tic-font /Fontsize tic-font-size def
	  /xlw  x x-with-k scaled-stringwidth def
	  /x0w _x0 cvi i2s scaled-stringwidth def  % XXX - xbias??
	  /x1w _x1 cvi i2s scaled-stringwidth def

	  x goodxtic
	  x   xlw x0w mid   _x0   within-n-of not
	  x   xlw x1w mid   _x1   within-n-of not
	  and and

	  % xlw x0w mid xlabstep lt and
	  % xlw x1w mid xlabstep lt and
	  % x xstep 10 mul mod 0 eq and
       end grestore
  } bdef

  /goodytic { % y - isgood
	  dup
	  _y0 sub abs ystep 10 div gt exch	% b x
	  _y1 sub abs ystep 10 div gt 	% b b
	  and				% b
  } bdef

  /goodylabel { % y - isgood
	% 2 was 10 below
	  dup _y0 sub abs ystep 2 div gt exch	% b x
	  dup _y1 sub abs ystep 2 div gt exch	% b b
	  ystep 5 mul mod 0 eq
	  and and
  } bdef

  want-htic {
    bordermatrix setmatrix % X-tics made in border matrix
    0 plot-gap neg translate
    4 dict begin
      /k-flag _x0 xbias sub 1000 mod 0 eq def
      _x0 _y0 2 copy xtic xlabel
      /k-flag _x1 xbias sub 1000 mod 0 eq def
      _x1 _y0 2 copy xtic xlabel
    end
    htic-step 0 eq {
      % automatic layout
      xstart xstep _x1 {
	  dup
	  dup goodxtic { _y0 xtic-small } { pop } ifelse
	  dup goodxlabel { _y0 xlabel+tic } { pop } ifelse
      } for
    } {
      % manual layout
      xbias 0 eq {
        xstart xstep _x1 { _y0 xtic-small } for
        xstart xlabstep _x1 {
		dup goodxlabel {
			dup tlab-mod mod 0 eq { _y0 xlabel+tic } {pop} ifelse
			%dup tlab-mod mod 0 eq { _y0 xlabel+tic } { _y0 xtic-small } ifelse
		}{ pop } ifelse
	} for
      } {
        xbias xstep _x1 { _y0 xtic-small } for
        xbias xstep neg _x0 { _y0 xtic-small } for
        xbias xlabstep _x1 {
		dup goodxlabel {
			dup xbias sub tlab-mod mod 0 eq { _y0 xlabel+tic } if
		} {
			pop
		} ifelse
	} for
        xbias xlabstep neg _x0 {dup xbias sub tlab-mod mod 0 eq { _y0 xlabel+tic } if } for
      } ifelse
    } ifelse
  } if

  want-vtic {
    bordermatrix setmatrix % Y-tics need new matrix
    plot-gap 0 translate
    inverted-numbers {
          _x0 _y0 translate 1 -1 scale 0 height neg translate
          _x0 neg _y0 neg translate } if
    _y0 _x1 2 copy ytic ylabel
    _y1 _x1 2 copy ytic ylabel
    vtic-step 0 eq {
      ystart ystep _y1 {
	  dup
	  dup goodytic { _x1 ytic-small } { pop } ifelse
	  dup goodylabel { _x1 ylabel+tic } { pop } ifelse
      } for
    } {
      ystart ystep yend { _x1 ytic-small } for
      ystart ylabstep yend { _x1 ylabel+tic } for
    } ifelse
  } if

  bordermatrix setmatrix
  Black
  gsave
} bdef

% ------------------------------------------------------------------------
% Tag file abbreviations

/Left /left def
/Undir /undir def
/Right /right def
/Sub -1 def
/Sup 1 def
/Plain 0 def

% extra kludges

/UnderlayRect { % a b UR
  8 dict begin
    /_yy y1 y0 sub def
    /_b exch def
    /_a exch def
    _a y0 _b y0 _yy 0 CFRect
  end
} def

/UnderlayRectBottomHalf { % a b UR
  8 dict begin
    /_yy y1 y0 sub 2 div def
    /_b exch def
    /_a exch def
    _a y0 _b y0 _yy 0 CFRect
  end
} def

/UnderlayRectTopHalf { % a b UR
  8 dict begin
    /_yy y1 y0 sub 2 div def
    /_b exch def
    /_a exch def
    _a y0 _yy add _b y0 _yy add _yy 0 CFRect
  end
} def

/extra-labels { plotdata-noclip /Fontsize 8 def } def

/center-y {
  _y1 _y0 mid
} bdef

/right-label { % y str
  _x1 0.3 inch add
  ticklen add
  side-label
} bdef

/center-left-label { % [...]
  center-y
  exch
  left-label
} bdef

/center-left-banner { % [...]
  4 dict begin
    /Fontsize banner-font-size def
    /Fontsize 13 def
    center-left-label
  end
} bdef

/left-label { % y str
  dup scaled-string-width neg
  _x0 add
  plot-gap sub
  ticklen sub
  side-label
} bdef

/side-label { % y str x
  3 2 roll % str x y
  moveto
  0 Fontsize 4 div scalefactor div inverted-dotplot not { neg } if rmoveto
  gsave
    savedmatrix setmatrix Hadjust { show-in-font } forall
  grestore
} bdef

/bottom-label { % x str
  _y0 0.25 inch sub
  3 -1 roll exch % str x y
  moveto
  ticklen 0 rmoveto
  0 Fontsize 4 div scalefactor div inverted-dotplot not { neg } if rmoveto
  gsave
    savedmatrix setmatrix Hadjust show-in-font
  grestore
} bdef

/bottom-label-rotated { % x str
  _y0 0.25 inch sub
  3 -1 roll exch % str x y
  moveto
  ticklen 0 rmoveto
  0 Fontsize 4 div scalefactor div inverted-dotplot not { neg } if rmoveto
  gsave
    savedmatrix setmatrix Hadjust 90 rotate show-in-font
  grestore
} bdef

/bottom-label-reverse-rotated { % x str
  _y0 0.25 inch sub
  3 -1 roll exch % str x y
  moveto
  ticklen 0 rmoveto
  0 Fontsize 4 div scalefactor div inverted-dotplot not { neg } if rmoveto
  gsave
    savedmatrix setmatrix Hadjust -90 rotate show-in-font
  grestore
} bdef

/GridLine { % y-percent --
  4 dict begin
    /p exch def
    /yp y1 p mul cvi def
    gsave
      [_x1 _x0 sub 500 div] 0 setdash
      _x0 yp _x1 yp 0.05 0.0 Line
    grestore
  end
} bdef

/HRule { % y
  plotdata-noclip
  4 dict begin
    /y exch def
    gsave
      [_x1 _x0 sub 500 div] 0 setdash
      _x0 plot-gap sub  y  _x1 plot-gap add  y  0.05 0.0 Line
    grestore
  end
  plotdata
} bdef

/SUA { % simple url annotation; x0 x1 string url -
  8 dict begin
    /u exch def /s exch def
    /right [[s /Helvetica]] u URLThinArrow
  end
} bdef

/SUB {% simple url box; x0 x1 url -
  8 dict begin
    /url_ exch def /x1_ exch def /x0_ exch def
    url-url
    gsave
    0.5 point setlinewidth
    %[x0_ current_row_base x1_ current_row_base thin add] url_ pdf-url
    [x0_ current_row_base x1_ current_row_base 0.5 point add] url_ pdf-url
    newpath x0_ current_row_base moveto x1_ current_row_base lineto stroke
    grestore
  end
} bdef

/SGB {% simple goto box; x0 x1 dest -
  8 dict begin
    /dest_ exch def /x1_ exch def /x0_ exch def
    gsave
    % XXX - must match elsewhere
    2.0 point setlinewidth
    [x0_ current_row_base 1.0 point sub
     x1_ current_row_base 1.0 point add] dest_ pdf-goto
    newpath x0_ current_row_base moveto x1_ current_row_base lineto stroke
    % 0.001 point setlinewidth 0 setgray
    % newpath x0_ current_row_base moveto x1_ current_row_base lineto stroke
    grestore
  end
} bdef

/KGB {% kludgy goto box; x0 x1 dest -
  8 dict begin
    /dest_ exch def /x1_ exch def /x0_ exch def
    /rect_ [x0_ current_row_base 0.5 point sub
            x1_ current_row_base 0.5 point add] def
    gsave
    0.5 point setlinewidth
    %[/Rect rect_ /Dest dest_ /Border [0 0 0.1] /Subtype /Link /ANN pdfmark
    %[/Rect rect_ /Border [0 0 0] /Action <</Subtype/GoTo /Dest dest_>> /Subtype /Link /ANN pdfmark

    [/Subtype /Widget
     /Rect rect_
     /F 4 /T (Radio) /FT /Btn /DA (/ZaDb 1 Tf 0 g) /Ff 49152 /AS /Off
     /MK <</CA (1)>>
     /AP <</N <<dest_ null>> >>
     /AA << /D <</Type/Action /S/GoTo /D dest_>> >>
    /ANN pdfmark

    newpath x0_ current_row_base moveto x1_ current_row_base lineto stroke
    grestore
  end
} bdef
%%% pdf support

%/define-distiller-ops {
%  % << /CompressPages false /CompatibilityLevel 1.3 >> setdistillerparams
%  [ /_objdef {afields} /type /array /OBJ pdfmark
%  [ /_objdef {aform} /type /dict /OBJ pdfmark
%  [ {aform} <<
%      /Fields {afields}
%      /DR << /Font << >> >>
%      /DA (/Helv 12 Tf 0 g)
%      /NeedAppearances true
%    >>
%  /PUT pdfmark
%
%  [ {Catalog} << /AcroForm {aform} >> /PUT pdfmark
%} bdef

/pdf-popup { % [ llx lly urx ury ] (text) 
  8 dict begin
    /text_ exch def
    /rect_ exch def
    /text_title gensym-title def
    /rect_title gensym-title def

    rect_ aload pop
    /ury exch def
    /urx exch def
    /lly exch def
    /llx exch def

    % popup text:
    [ /Subtype /Widget
      /Rect [ llx lly
              llx text_ scaled-stringwidth add
              lly (M)   scaled-stringwidth add
            ] 
      /F  3 % makes the field HIDDEN
      /T  text_title
      /FT /Tx
      /DA (/Helv 14 Tf 0 0 0 rg)
      /V  text_
    /ANN pdfmark

    % rollover area:
    [ /Subtype /Widget
      /Rect rect_
      /F  4
      /T  rect_title
      /Ft /Btn
      /AA <<
        /E << /S /Hide /T text_title /H false >>
        /X << /S /Hide /T text_title /H true  >>
      >>
    /ANN pdfmark
  end 
} bdef

/tooltip { % x0 x1 (string)
  4 dict begin
    /str_ exch def
    /x1_ exch def
    /x0_ exch def
    % [x0_ _y0 x1_ _y1] str_ pdf-popup
    [x0_ _y0 x1_ _y1] str_ pdf-url % sigh...
  end
} bdef

/pdf-url { % [ llx lly urx ury ] (http)
  2 dict begin
    /http_ exch def
    /rect_ exch def

    [/Rect rect_
     %/Border [1 1 0]
     /Border [0 0 0]
     /Color [0 0 1]
     /Action << /Subtype /URI   /URI http_ >>
     /Subtype /Link
    /ANN pdfmark
  end
} bdef

/pdf-dest-thispage { % name -
    1 dict begin /dest_ exch def
    [/Dest dest_ /View [/FitB] /DEST pdfmark
    end
} bdef

/pdf-dest-here { % name -
  1 dict begin /dest_ exch def [ /Dest dest_ /DEST pdfmark end
} bdef

/pdf-dest-here-str3 { % (s) (s) (s)
  str3name pdf-dest-here
} bdef

/pdf-goto { % [ llx lly urx ury ] /dest
  2 dict begin
    /dest_ exch def
    /rect_ exch def

    [/Rect rect_ /Border [0 0 0] /Action <</Subtype/GoTo /Dest dest_>> /Subtype /Link /ANN pdfmark
    %[/Rect rect_ /Dest dest_ /Border [0 0 0.1] /Subtype /Link /ANN pdfmark
  end
} bdef

% x0 y0 /dir str url
/url-box-params {
    /url_ exch def
    parameter-type-kludge
    /strlist_ exch def
    /dir_ exch def
    /x1_ exch def
    /x0_ exch def
    /Fontsize label-font-size def
    /strlen_ strlist_ scaled-string-width def
} bdef

/url-url {
	/url_ URL-base url_ concatstrings def
    } bdef

/v-url-box {
  8 dict begin
    url-box-params
    url-url
    [x0_ current_row_base x1_ text_bottom strlen_ add] url_ pdf-url
    x0_ x1_ dir_ strlist_
  end
} bdef

/h-url-box {
  8 dict begin
    url-box-params
    url-url
    [x0_ current_row_base x1_ box_top row_gap add] url_ pdf-url
    x0_ x1_ dir_ strlist_
  end
} bdef

/URL-annot-maxr { pop } def

/URL-annot {
  % 33000 33500 Blue (string) (url) 3 URL-annot
  9 div % XXX - depends on defines from elsewhere
    /URL-annot-base where {pop URL-annot-base} {2.0} ifelse
    add 0.1 add setrow
  exch pop
  SUB
} bdef

/str3name {
  3 dict begin
    /r exch def /b exch def /a exch def
    a (_) concatstrings
    b (_) concatstrings
    concatstrings r concatstrings cvn 
  end
} bdef

/int3name {
  3 dict begin
    /r exch def /b exch def /a exch def
    a i2s b i2s r i2s str3name
  end
} bdef

/annot-link { % 33000 33500 3  -
  4 dict begin
    /r exch def /b exch def /a exch def

    r 8 div % /9
    /URL-annot-base where {pop URL-annot-base} {2.0} ifelse
    add 0.1 add setrow

    a b
    a b r int3name
    SGB
  end
} bdef

/nannot-link { % start end row nth -
  4 dict begin
    /n exch def /r exch def /b exch def /a exch def

    r 8 div % /9
    /URL-annot-base where {pop URL-annot-base} {2.0} ifelse
    add 0.1 add setrow

    a b
    (annot.) n i2s concatstrings cvn
    SGB
  end
} bdef


/bkmrk-annot-link { % start end row (outline) - 
    cvn pdf-dest-thispage
    annot-link
} bdef

/bkmrk-nannot-link { % start end row nth (outline) -
    cvn pdf-dest-thispage
    nannot-link
    Black
} bdef

end %DOTPLOT
%%EndProcSet
@
}

while
 case "$1" in
   -auto) NO_AUTO="false" ;;
   -bannerfontsize=*) BANNERFONTSIZE=`optint "$1"`;;
   -creator=*) CREATOR=`optstr "$1" | tr -d '\012\015'`;;
   -eps) EPSFLAG="true";;
   -h=*) OVERRIDE_HEIGHT=`optint "$1"`;;
   -hl=*) HLAB=`optint "$1"` ;;
   -ht=*) HTIC=`optint "$1"` ;;
   -i) INVERTED_NUMBERS="true" INVERTED_FEATURES="true" INVERTED_PLOT="true";;
   -if) INVERTED_FEATURES="true";;
   -in) INVERTED_NUMBERS="true";;
   -inv) INVERTED_NUMBERS="true" INVERTED_FEATURES="true" INVERTED_PLOT="true";;
   -ip) INVERTED_PLOT="true";;
   -kflag) KFLAG="true" ;;
   -labelfontsize=*) LABELFONTSIZE=`optint "$1"`;;
   -landscape) ORIENTATION=1;;
   -nb) BANNER="false" ;;
   -nc) CLIP_FEATURES="false" ;;
   -nh) WANT_HTIC="false" ;;
   -noauto) NO_AUTO="true" ;;
   -nokflag) KFLAG="false" ;;
   -nopercent) PERCENT="false" ;;
   -noprocset) NOPROCSET=1;;
   -noshowpage) NOSHOWPAGE=1;;
   -nv) WANT_VTIC="false" ;;
   -nw) WANT_HTIC="false" ;;
   -percent) PERCENT="true" ;;
   #-plotgap=*) PLOTGAP=`optint "$1"`;;
   -plotgap*) PLOTGAP=0;;
   -portrait) ORIENTATION=2;;
   -ppi=*) OVERRIDE_SCALE=`optint "$1"`;;
   -procset) emitprocset; exit 0;;
   -strictfeatures) STRICT_FEATURES="true";;
   -strictlabels) STRICT_LABELS="true";;
   -ticfontsize=*) TICFONTSIZE=`optint "$1"`;;
   -tl=*) TLAB=`optint "$1"` ;;
   -url=*) URL_BASE=`optstr "$1"`;;
   -vsf=*) VSCALEFUDGE=`optint "$1"`;;
   -vt=*) VTIC=`optint "$1"` ;;
   -w=*) OVERRIDE_WIDTH=`optint "$1"`;;
   -x=*) XOFFSET=`optint "$1"` ;;
   -xbias=*) XBIAS=`optint "$1"`;;
   -xticbias=*) XTICBIAS=`optint "$1"`;;
   -y=*) YOFFSET=`optint "$1"` ;;
   -yl=*) VLAB=`optint "$1"` ;;
   -[a-zA-Z]*) echo "unknown flag $1" 1>&2 ;;
   *) false ;;
 esac
do shift
done

case $# in
0|4) ;;
*) echo $USAGE 1>&2 ;
   echo "$1 -- unknown" 1>&2;
   exit 1;;
esac

cat <<@
%!PS-Adobe-3.0 EPSF-3.0
%%DocumentFonts: Times-Roman Courier Symbol Helvetica
%%DocumentProcSets: PSU_dotplot 1.0 0.0
%%Title: alignment
%%Creator: $CREATOR; $RCSID
%%CreationDate: `date`
%%Copyright: software and libraries, Scott Schwartz, 1999
%%For: ${LOGNAME:-${USER:-'nobody'}}
%%Pages: 1
%%BoundingBox: 0 0 612 792
%%EndComments
@

cat <<'@'
gsave
@

[ x"$NOPROCSET" != "x1" ] && emitprocset

cat <<'@'
%%EndProlog
%%BeginSetup
%%EndSetup

@

## Page setup
cat <<@
%%Page: 1 1
%%BeginPageSetup
% neutralize pdfmark
/pdfmark where {pop} {userdict /pdfmark /cleartomark load put} ifelse
DOTPLOT begin
% thin setlinewidth % ??
%%EndPageSetup

% variables
/need-main true def
/need-report-plot false def
/Vundershoot false def
/Vovershoot false def
/Hundershoot false def
/Hovershoot false def
/current_row_base 0 def
/box_adjustment 0 def
/current-mode /none def
/units-per-inch 72 def
/scalefactor 1 def

/banner-font-size 10 def
/label-font-size 7 def
/Fontsize 7 def
/basic-arrow-gap 0 def

% defaults
/inverted-dotplot false def /inverted-features false def
/inverted-numbers false def /clip-features true def
/noauto true def
/banner-line true def /description () def /date (`date`) def
/current-directory (`(pwd|sed -e 's;^/tmp_mnt;;' -e 's;^/auto;;') 2>/dev/null`) def
/override-width 0 def /override-height 0 def /override-scale 0 def
/want-htic true def /want-vtic true def
/htic-step 0 def /vtic-step 0 def
/hlab-step 0 def /vlab-step 0 def
/tlab-mod 1 def
/y-scale-fudge 1 def
/x-offset 0 def /y-offset 0 def
/page-orientation 0 def
/tic-font /Helvetica def
/tic-font-size 7 def
/strict-features false def
/strict-labels false def
/percent-flag false def
/k-flag true def
/xbias 0 def
/xticbias 0 def
/plot-gap-pt 2 def
/x0 0 def
/x1 1000 def
/y0 0 def
/y1 1000 def
/URL-base () def
/eps-flag false def

% ...in points
/feature-line-width 0.5 def
/feature-circle-radius 1.5 def

/user-options {
	${BANNER+/banner-line $BANNER def}
	${BANNERFONTSIZE+/banner-font-size $BANNERFONTSIZE def}
	${CLIP_FEATURES+/clip-features $CLIP_FEATURES def}
	${EPSFLAG+/eps-flag $EPSFLAG def}
	${HLAB+/hlab-step $HLAB def}
	${HTIC+/htic-step $HTIC def}
	${INVERTED_FEATURES+/inverted-features $INVERTED_FEATURES def}
	${INVERTED_NUMBERS+/inverted-numbers $INVERTED_NUMBERS def}
	${INVERTED_PLOT+/inverted-dotplot $INVERTED_PLOT def}
	${KFLAG+/k-flag $KFLAG def}
	${LABELFONTSIZE+/label-font-size $LABELFONTSIZE def}
	${NO_AUTO+/noauto $NO_AUTO def}
	${ORIENTATION+/page-orientation $ORIENTATION def}
	${OVERRIDE_HEIGHT+/override-height $OVERRIDE_HEIGHT def}
	${OVERRIDE_SCALE+/override-scale $OVERRIDE_SCALE def}
	${OVERRIDE_WIDTH+/override-width $OVERRIDE_WIDTH def}
	${PERCENT+/percent-flag $PERCENT def}
	${PLOTGAP+/plot-gap-pt $PLOTGAP def}
	${STRICT_FEATURES+/strict-features $STRICT_FEATURES def}
	${STRICT_LABELS+/strict-labels $STRICT_LABELS def}
	${TICFONTSIZE+/tic-font-size $TICFONTSIZE def}
	${TLAB+/tlab-mod $TLAB def}
	${URL_BASE+/URL-base $URL_BASE def}
	${VLAB+/vlab-step $VLAB def}
	${VSCALEFUDGE+/y-scale-fudge $VSCALEFUDGE def}
	${VTIC+/vtic-step $VTIC y-scale-fudge mul def}
	${WANT_HTIC+/want-htic $WANT_HTIC def}
	${WANT_VTIC+/want-vtic $WANT_VTIC def}
	${XBIAS+/xbias $XBIAS def}
	${XOFFSET+/x-offset $XOFFSET inch def}
	${XTICBIAS+/xticbias $XTICBIAS def}
	${YOFFSET+/y-offset $YOFFSET inch def}
	${1+/x0 $1 def /x1 $2 def /y0 $3 def /y1 $4 def}
} def
@

# set file options
# main
# plotdata
# ... lines
# tag definitions
# horizontal
# ... stuff
# vertical
# ... stuff
cat

cat <<'@'
/x-axis-label where { pop horizontal 1 setrow x-axis-label Hbanner } if
/y-axis-label where { pop vertical 1 setrow y-axis-label Vbanner } if
grestore
%%PageTrailer
end %DOTPLOT
grestore
%%Trailer
@
#case "$ORIENTATION" in
#1) echo '%%Orientation: Landscape';;
#2) echo '%%Orientation: Portrait';;
#esac

case "$NOSHOWPAGE" in
'') echo "showpage"
esac
#echo '%%EOF'
exit 0
