/* mlav.h - header file of procedures for reading compressed multiple alignment
*  files, as generated by the "refine" program
*/

/* In our way of storing multiple alignments, each sequence has sequence of
*  "blocks", with a positive integer signifying the length of a run of
*  consecutive sequence entries, and an negative number signifying the length
*  of a gap
*/
typedef struct blk {
	int size;
	struct blk *next;
} Block, *Block_ptr;

/* A sequence comes with two extra values, "cur" and "scan", used as scratch
*  areas by some programs.
*/
typedef struct mseq {
	char *name, *s;
	int beg, end, cur;
	Block_ptr block, scan;
} Mseq;

/* read_mseqs returns an array of Mseqs (through an argument) and the number
*  of sequences (procedure value)
*/
int read_mseqs(char *filename, Mseq **mseq);

/* malign_matrix returns a textual form of the alignment; the alignment must
* be "single coverage", in the following sense.  Consecutive aligned
*  sequences that have the same "root name" (after removing a trailing "-",
*  which denotes reverse complement, and any letters starting with the first
*  ":", which is followed by a contig name) are taken to be from the same
*  species.  Single coverage means that any position in the first sequence can
*  align with at most one position in each other species.
*/
char **malign_matrix(char *filename, int *nrow_ptr, int *ncol_ptr,
  char ***seq_name, char unaligned);
