/* overview - prepare a command that makes an overview of the pip */

#include "util.h"
#include "lav.h"

#define TMP_A "out.ov.a"
#define TMP_H "out.ov.h"
#define TMP_V "out.ov.v"
#define TMP_U "out.ov.u"

#define MIN_LEN 100
#define MIN_PCT 70

static const char Align[] =
"#:lav\n\
d {\n\
  \"overview\"\n\
}\n\
s {\n\
  \"x\" 1 1\n\
  \"y\" 1 1\n\
}\n\
h {\n\
   \"\"\n\
   \"\"\n\
}\n\
a {\n\
  s 0\n\
  b 1 2\n\
  e 1 2\n\
}\n";

#define Cmd_tag \
"dottag1 %s %s S=1 | sed -e \"/0 setrow/s//-0.2 setrow/\" > %s\n"

#define Cmd_und \
"mkunderlays %s > %s\n"

static const int t[] =
{100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000, 100000, 200000, 500000,
1000000000};

static int notreally = 0;  /* print commands without executing them */

static int vlen(char **v)
{
    int n = 0;
    if (!v) return 0;
    while (v[n]) ++n;
    return n;
}

/* XXX -- should be in a pip library */
static void check_safechars(char *s)
{
    if (!s)
	return;
    for ( ; *s; ++s) {
	if (!isascii(*s) ||
	    (!isalnum(*s) && (*s != '.') && (*s != '-') && (*s != '_')))
		fatalf("invalid character '%c' (%d)\n", *s, *s); 
		/*  *s = "_";  */
    }
}

static void systemf(const char *fmt, ...)
#ifdef __GNUC__
      __attribute__ ((format (printf, 1, 2)))
#endif
    ;

/* XXX - this used to be in a library */
/* it is in util.c in the piplib dir */
/* gcc < 4 used to use whichever declaration was last. */
/* gcc >= 4.0 now sees this as an error */
/*
static void fatalfr(const char *fmt, ...)
{   
    va_list ap;
    va_start(ap, fmt); 
    fflush(0);
    print_argv0();
    (void)vfprintf(stderr, fmt, ap);
    (void)fprintf(stderr, ": %s\n", strerror(errno));
    va_end(ap);
    exit(1);
}
*/

/* XXX -- should be in a library */
static char *file2string(const char *name)
{
    int n;
    FILE *fp = ckopen(name, "r");
    char *s = ckalloc(512);
    /* XXX - should use a growable array here */
    /* XXX - instead just read the beginning */
    if (fgets(s, 512, fp) == 0)
	fatalfr("fgets");
    fclose(fp);

    n = strlen(s);
    if (n > 0) s[n-1] = 0;
    return s;
}

/* XXX -- should be in a library */
static void systemf(const char *fmt, ...)
{       
    int n;
    va_list ap;
    char buf[4096];

    fflush(0);
    va_start(ap, fmt);
    n = vsnprintf(buf, sizeof(buf), fmt, ap);
    va_end(ap);
    if (n < 0 || n > 4096)
	fatalf("vsnprintf: n==%d\n", n);

    if (notreally)
	(void)fprintf(stderr, "%s\n", buf);
    else if (system(buf) != 0)
	fatalf("system: %d\n", n);
}       

static void make_htag(const char *exons, char *align[], int n, const char *out)
{
    if (exons && align && align[0] && align[n])
	systemf(Cmd_tag, exons, align[n], out);
    else
	fclose(ckopen(out, "w"));
}

char *seqname(char *file[], int c)
{
    char *num = ckalloc(64);  /* XXX - leak */
    char *s;

    snprintf(num, 64, "%d", c+1);
    s = (file && file[0] && file[c]) ? file2string(file[c]) : num;
    check_safechars(s);
    return s;
}

static void make_vtag(char *s, const char *out)
{
    FILE *fp = ckopen(out, "w");
   
    /* like mpm_rowtag */
    fprintf(fp, "extra-labels\n");
    fprintf(fp, "[[(%s) /Helvetica 8]] center-left-banner\n", s);
    fclose(fp);
}

int get_tic(int seq_len)
{
	int i;

	for (i = 0; i < 12; ++i)
		if (10*t[i] > seq_len)
			break;
	return t[i];
}

enum { NoTag=0, Tag=1, NoTic=0, Tic=1 };

void dopip(const char *u, const char *a, const char *h, const char *v,
	    int Y, int seq_len, int needtag, int needtic)
{
    systemf("pmps -Y %d -w 1 -utag %s -align %s "
	    " -vtag %s -htag %s "
	    " -dot -eps -nb -nv -portrait -plotgap=0 -noshowpage "
	    " -labelfontsize=9 -strictlabels "
	    " %s -ht=%d -ticfontsize=9 "
	    " -y=9.5 1 %d %d %d ",

	     10*Y, u, a, v,
	     needtag ? h : "/dev/null",
	     needtic ? "" : " -nh ",
	     get_tic(seq_len),
	     seq_len, 2*Y, 4*Y);
}


void new_ul(char *infile, char *outfile, char *name)
{
	int b, e, p;
	FILE *fp, *op;
	char buf[500];

	fp = ckopen(infile, "r");
	fgets(buf, 500, fp);
	if (!same_string(buf, "#:lav\n"))
		fatalf("'%s' is not an lav file", infile);
	op = ckopen(outfile, "w");
	fprintf(op, "# underlay for %s generated by overview\n\n", name);
	while (fgets(buf, 500, fp))
		if (same_string(buf, "a {\n")) {
			fgets(buf, 500, fp); /* score */
			fgets(buf, 500, fp);
			if (sscanf(buf, "  b %d", &b) != 1)
				fatalf("begin: %s", buf);
			fgets(buf, 500, fp);
			if (sscanf(buf, "  e %d", &e) != 1)
				fatalf("end: %s", buf);
			fprintf(op, "%d %d Green\n", b, e);
		}
	rewind(fp);
	while (fgets(buf, 500, fp))
		if (sscanf(buf, "  l %d %*d %d %*d %d", &b, &e, &p) == 3 &&
		    e-b+1 >= MIN_LEN && p >= MIN_PCT)
			fprintf(op, "%d %d Red\n", b, e);
	fclose(op);
	fclose(fp);
}

int main(int argc, char **argv)
{
	int seq_len, Y;
	int c, err=0;
	FILE *fp;
	char *exons=0, *underlay=0;
	char **align=0;
	char **name=0;
	int nalign, nname;
	char **ap;
	int has_ulay=0;
	char tmp1[100];

	argv0 = "overview";

	while ((c = getopt(argc, argv, "ne:u:")) != -1)
	switch (c) {
	case 'n': notreally = 1; break;
	case 'e': exons = optarg; break;
	case 'u': underlay = optarg; break;
	case '?': err++; break;
	}
	if (err || argc-optind < 1)
	    fatal("usage: -u underlay -e exons -- blastz ... -- name ...");

	check_safechars(exons);
	check_safechars(underlay);
	for (c=optind; c < argc; c++) check_safechars(argv[c]);

	/* point align at start of alignment files,
	 * point name at start of name files,
	 * replace separating "--" with 0 in argv. */
	align = ap = argv+optind;
	while (ap && ap[0] && strcmp(ap[0], "--"))
	    ++ap;
	if (ap && ap[0] && ap[0][0] == '-') {
	    ap[0] = 0;
	    name = ap+1;
	} else {
	    name = ap;
	}
	assert(align);
	assert(name);

	nalign = vlen(align);
       	nname = vlen(name);
	if (nname && nname != nalign+1)
	    fatalf("mismatched number of alignments and names");

	seq_len = seq1len(align[0]);
	if (seq_len < 1000) seq_len = 1000;
	Y = seq_len/100;
	if (Y==0) Y = 1;

	if (underlay && (fp = ckopen(underlay,"r")) != NULL && getc(fp) != EOF) {
		fclose(fp);
		has_ulay = 1;
	}

	if (!notreally) {
	    fp = ckopen(TMP_A, "w");
	    fprintf(fp, Align);
	    fclose(fp);
	}

	puts("%!PS-Adobe-3.0\n");
	systemf("%s", "pdf-header");
	puts("[/Title(overview)/Dest[{ThisPage}/FitB]/OUT pdfmark\n");

	if (has_ulay) {
	    make_vtag("underlay", TMP_V);
	    make_htag(exons, align, 0, TMP_H);
	    systemf(Cmd_und, underlay, TMP_U);
	    dopip(TMP_U, TMP_A, TMP_H, TMP_V, Y, seq_len, Tag, NoTic);
	}

	for (c=0; c<nalign; ++c) {
	    puts("0 -10 translate\n");

	    make_vtag(seqname(name, c+1), TMP_V);
	    make_htag(exons, align, c, TMP_H);

		sprintf(tmp1, "seq%02d.ov.ulay", c+2);
	    new_ul(align[c], tmp1, seqname(name, c+1));
	    systemf(Cmd_und, tmp1, TMP_U);

	    dopip(TMP_U, TMP_A, TMP_H, TMP_V, Y, seq_len,
		  (c==0 && !has_ulay), (c==nalign-1));
	}
	puts("showpage\n");

	return 0;
}
