/* seqmask -- convert identified repeat elements to lower case */
/* simple or low-complexity sequences, but not RNAs, are also masked */

#include "util.h"
#include "seq.h"

enum {
  LINE_LEN=50,
  BUF_SIZE=1000
};

int main(int argc, char **argv)
{
	FILE *fp;
	SEQ *sp;
	int N, beg, end, i, hum_mus = 0;
	uchar *seq;
	char buf[BUF_SIZE], *p;

	argv0 = "seqmask";
	if (argc == 4 && strcmp(argv[1],"-hum+mus") == 0)
		hum_mus = 1;
	else if (argc != 3)
		fatalf("args = [-hum+mus] seq_file \"repeats_tag\"-outfile");

	sp = seq_get(argv[argc-2]);
	if (sp == NULL)
		fatal("First sequence given in improper format.");
	N = SEQ_LEN(sp);
	if ((SEQ_FROM(sp)) != 1)
		fatal("masking of regions in a sequence is not implemented");
	seq = SEQ_CHARS(sp);

	fp = ckopen(argv[argc-1], "r");
	if (fgets(buf, BUF_SIZE, fp) && !same_string(buf, "%:repeats\n"))
		fatal("Not a valid repeats file.");

	printf(">%s", argv0);
	for (i = 1; i < argc; ++i)
		printf(" %s", argv[i]);
	putchar('\n');

	while (fgets(buf, BUF_SIZE, fp)) {
		if (sscanf(buf, "%d %d", &beg, &end) != 2)
			fatalf("%s: conversion failed", buf);
		if (end > N) {
			if ((p = strchr(buf, '\n')) != NULL)
				*p = '\0';
			fprintf(stderr, "\nSequence length is %d,", N);
			fprintf(stderr, " whereas masking is at \"%s\"\n",
				buf);
			fatal("Error in first sequence or RepeatMasker file.");
		}
		if (beg > end || beg < 1 || end > N)
			fatalf("illegal positions: %s", buf);
		if (hum_mus && (strstr(buf, "Alu") || strstr(buf, "B1")))
			for (i = beg-1; i < end; ++i)
				seq[i] = 'X';
		else if (!strstr(buf, "RNA"))
			for (i = beg-1; i < end; ++i)
				seq[i] = tolower(seq[i]);
	}
	for (i = 1; i <= N; ++i) {
		putchar(seq[i-1]);
		if (i % LINE_LEN == 0)
			putchar('\n');
	}
	if (i % LINE_LEN != 1)
		putchar('\n');

	seq_close(sp);
	fclose(fp);
	return(0);
}
