package Pipmisc;
use strict;
use warnings;
use Net::DNS::Resolver;

BEGIN {
    use Exporter   ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # must be all one line, for MakeMaker
    $VERSION=do{my@r=(q$Revision: 1.00 $ =~/\d+/g);sprintf"%d."."%02d"x$#r,@r};
    
    @ISA         = qw(Exporter);
    @EXPORT      = qw(
	&cgierr_exit &notify &err &stash_err &cgierr &eupload &eparam
	&canonicalize &save_query &copyfile &copystring &copyfs &nsubdirs
	&check_queue_size &valid_smtp_host &check_email &check_strlen
	&check_nonempty &check_lowercase &check_server_status &check_lint
	&client_id &file_lines &V &untaint &YN
    );

    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],
    
    # your exported package globals go here,
    # as well as any optionally exported functions
    @EXPORT_OK   = qw($Var1 %Hashit &func3);
}
our @EXPORT_OK;

END { }

sub cgierr_exit {
    my $q = shift;
    &cleanup;
    print $q->end_html;
    exit 0;
}

sub notify {
    my $s = shift;
    print $s;
    print STDERR $s;
}

sub err {
    my $q = shift;
    my $e = shift;

    # warn($e . "\n"); # for log
    print $q->header,
	$q->start_html('Problems'),
	$q->h2('Request not processed'),
	$q->strong($e);
    cgierr_exit($q);
}

sub stash_err {
    my $q = shift;
    my $d = shift;
    my $s = shift;
    open(F, ">>$d/ERR0") or die "open: $!";
    print F "\n$s\n";
    close(F);
    err($q, $s);
}

sub cgierr {
	my $q = shift;
	my $e = $q->cgi_error;
	print $q->header,
		$q->start_html('Problems'),
		$q->h2('Request not processed'),
		$q->strong($e);
	cgierr_exit($q);
}

sub eupload {
	my $q = shift;
	my $a = shift;
	my $f = $q->upload($a);
	cgierr($q) if (!$f && $q->cgi_error);
	 # linux/perl bug??
         if (defined($f) && -f $f && -B $f) {
             err($q, "$a is not a plain text file");
         }
	# local($,)=" "; print "$a $f "; print stat $f; print "<br>\n";
	return $f;
}

sub eparam {
	my $q = shift;
	my $a = shift;
	my $f = $q->param($a);
	cgierr($q) if (!$f && $q->cgi_error);
	if ($f) {
		for ($f) {
			s/^\s+//;
			s/\s+$//;
		}
        }
	return $f;
}

sub canonicalize {
	# only handles non-pathalogical cases well.
	for (@_) { 
                s/\r\n/\n/g;			# dos -> unix
                s/\r/\n/g;			# mac -> unix
                s/[ \t]+$//gm;			# trim trailing whitespace
		$_ .= "\n" unless $_ =~ /\n$/;	# make sure last line is ok
	}
}

sub save_query {
	my $q = shift;
	my $file = shift;

	die "Undefined file" unless $file;
	open(F, ">$file") or die "Cannot open $file: $!";
	$q->save(\*F);
	close(F);
}

sub copyfile {
	my $fh = shift;
	my $file = shift;

	die "Undefined file" unless $file;
	open(F, ">$file") or die "Cannot open $file: $!";
	if (defined($fh)) {
		seek($fh,0,0); # workaround bug in CGI::upload
		while (<$fh>) {
			canonicalize($_);
			print F or die "Cannot write $file: $!" 
		}
	}
	close(F) or die "Cannot close $file: $!";
}

sub copystring {
	my $str = shift;
	my $file = shift;

	die "Undefined file" unless $file;
	$str = "" unless defined($str);
	open(F, ">$file") or die "Cannot open $file: $!";
	canonicalize($str);
	print F $str or die "Cannot write $file: $!";
	close(F) or die "Cannot close $file: $!";
}

sub copyfs {
    my $str = shift;
    my $fh = shift;
    my $file = shift;

    if (defined($str) && ($str !~ /^\s*$/)) {
        copystring($str, $file);
    } else {
        copyfile($fh, $file);
    }
}

sub nsubdirs {
  my($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
     $atime,$mtime,$ctime,$blksize,$blocks) = stat(shift);
  return (-d _ ? $nlink-2 : 0);
}

sub check_queue_size {
  my $q = shift;
  my $tmp = shift;
  my $n = 0;
  err $q, "Too many pending submissions.\n"
    if nsubdirs("$tmp/tmp") > 50;
  err $q, "Too many submissions in the new-work queue.\n"
    if ($n = nsubdirs("$tmp/new")) > 50;
  return $n;
}

sub valid_smtp_host {
  my $name = shift;
  my $resolver = new Net::DNS::Resolver;

  return 1 unless $resolver; # XXX - fail soft.
  return defined($resolver->query($name, "MX", "IN"))
      || defined($resolver->query($name, "A",  "IN"));
}

sub check_email {
    my $q = shift;
    my $n = shift;
    my $p = shift;
    err($q, "Parameter <i>$n</i> is empty.")
        if (!defined($p) || length($p) == 0);
    err($q, "Parameter <i>$n</i> isn't of the form user\@host.domain: <i>$p</i>.")
        unless $p =~ /^[[:graph:]]+@([[:graph:]]+\.[[:graph:]]+)$/;
    err($q, "Host <i>$1</i> doesn't have an A or MX record; is it misspelled?")
        unless valid_smtp_host $1;
}

sub check_strlen {
    my $q = shift;
    my $n = shift;
    my $p = shift;
    err($q, "Parameter <i>$n</i> is empty.") if (length($p) == 0);
}

sub check_nonempty {
    my $q = shift;
    my $d = shift;
    my $f = shift;
    my $g = "$d/$f";
    stash_err($q, $d, "Data file <i>$f</i> is empty.") if (-z $g);
}

sub check_lowercase {
    my $q = shift;
    my $d = shift;
    my $f = shift;
    my $g = "$d/$f";
    open(F, "$g") or die "Cannot open $g: $!";
    while (<F>) {
        stash_err($q, $d, "Data file $f contains lower case (masked out) letters.")
	    if (/^[A-Za-z]/ && /[a-z]/);
    }
    close(F) or die;
}

sub check_server_status {
    my $q = shift;
    my $run = shift;
    system("pipserver-status") == 0
	or err($q, "Server is down now.\n");
}

sub check_lint {
    my $q = shift;
    my $t = shift;
    my $res = qx{
      cd $t &&
      pipenv pip-lint \\
	--seq1 ./seq1data \\
	--seq2 ./seq2data \\
	--exon ./exons \\
	--rpts ./seq1mask \\
	--ulay ./colors \\
	--nvce ./novice \\
	--gxan ./genxan \\
	--anno ./annotations
    };
    $res = "status=$?:$!:$@:" unless defined($res);
    stash_err($q, $t, "<pre>$res</pre>") if ($? != 0);
}

sub client_id {
    my $ra = $ENV{'REMOTE_ADDR'} || "";  $ra =~ tr{\012\015\n\r}{    };
    my $rh = $ENV{'REMOTE_HOST'} || "";  $rh =~ tr{\012\015\n\r}{    };
    my $ru = $ENV{'REMOTE_USER'} || "";  $ru =~ tr{\012\015\n\r}{    };
    my $ri = $ENV{'REMOTE_IDENT'} || ""; $ri =~ tr{\012\015\n\r}{    };
    my $at = $ENV{'AUTH_TYPE'} || "";    $at =~ tr{\012\015\n\r}{    };

    my $id = "REMOTE_ADDR=$ra\n" 
           . "REMOTE_HOST=$rh\n"
           . "REMOTE_USER=$ru\n" 
           . "REMOTE_IDENT=$ri\n"
           . "AUTH_TYPE=$at\n";

    return $id;
}

sub file_lines {
    # man perlfaq5
    my $filename = shift;
    my $lines = 0;
    my $buffer = "";
    open(FILE, $filename) or die "Can't open `$filename': $!";
    while (sysread FILE, $buffer, 4096) {
        $lines += ($buffer =~ tr/\n//);
    }
    close FILE;
    return $lines;
}

sub V {
    my $v = shift;
    return $v if defined($v);
    return "";
} 

sub untaint {
	$_[0] =~ m/^(.*)$/so;
	$1;
}

sub YN {
    return (V($_[0]) =~ /1/) ? "yes" : "no";
}

1;

